<?php

/**
 * @file
 * Implementation of the PAD File forms.
 */

/**
 * Retrieve the list of padfiles defined at this site.
 *
 * \return An array composed of the raw rows from the database.
 */
function _padfile_get_padfiles() {
  // query with a default order
  $padfiles = array();
  $result = db_query("SELECT * FROM {padfile} ORDER BY program_name");
  while ($row = db_fetch_object($result)) {
    $padfiles[] = $row;
  }
  return $padfiles;
}

/**
 * Retrieve the list of descriptions for the specified PAD File.
 *
 * \return An array composed of the raw rows from the database.
 */
function _padfile_get_descriptions($padid) {
  // query with a default order
  $descriptions = array();
  $result = db_query("SELECT * FROM {padfile_descriptions} WHERE padid = %d ORDER BY language", $padid);
  while ($row = db_fetch_object($result)) {
    $descriptions[] = $row;
  }
  return $descriptions;
}




function _padfile_reindex($in, &$out) {
  if (!$out) {
    foreach ($in as $k) {
      $out[$k] = $k;
    }
  }

  return $out;
}


function _padfile_categories_array() {
  static $cats = array(
    'Audio & Multimedia::Audio Encoders/Decoders',
    'Audio & Multimedia::Audio File Players',
    'Audio & Multimedia::Audio File Recorders',
    'Audio & Multimedia::CD Burners',
    'Audio & Multimedia::CD Players',
    'Audio & Multimedia::Multimedia Creation Tools',
    'Audio & Multimedia::Music Composers',
    'Audio & Multimedia::Other',
    'Audio & Multimedia::Presentation Tools',
    'Audio & Multimedia::Rippers & Converters',
    'Audio & Multimedia::Speech',
    'Audio & Multimedia::Video Tools',
    'Business::Accounting & Finance',
    'Business::Calculators & Converters',
    'Business::Databases & Tools',
    'Business::Helpdesk & Remote PC',
    'Business::Inventory & Barcoding',
    'Business::Investment Tools',
    'Business::Math & Scientific Tools',
    'Business::Office Suites & Tools',
    'Business::Other',
    'Business::PIMS & Calendars',
    'Business::Project Management',
    'Business::Vertical Market Apps',
    'Communications::Chat & Instant Messaging',
    'Communications::Dial Up & Connection Tools',
    'Communications::E-Mail Clients',
    'Communications::E-Mail List Management',
    'Communications::Fax Tools',
    'Communications::Newsgroup Clients',
    'Communications::Other Comms Tools',
    'Communications::Other E-Mail Tools',
    'Communications::Pager Tools',
    'Communications::Telephony',
    'Communications::Web/Video Cams',
    'Desktop::Clocks & Alarms',
    'Desktop::Cursors & Fonts',
    'Desktop::Icons',
    'Desktop::Other',
    'Desktop::Screen Savers: Art',
    'Desktop::Screen Savers: Cartoons',
    'Desktop::Screen Savers: Nature',
    'Desktop::Screen Savers: Other',
    'Desktop::Screen Savers: People',
    'Desktop::Screen Savers: Science',
    'Desktop::Screen Savers: Seasonal',
    'Desktop::Screen Savers: Vehicles',
    'Desktop::Themes & Wallpaper',
    'Development::Active X',
    'Development::Basic, VB, VB DotNet',
    'Development::C / C++ / C#',
    'Development::Compilers & Interpreters',
    'Development::Components & Libraries',
    'Development::Debugging',
    'Development::Delphi',
    'Development::Help Tools',
    'Development::Install & Setup',
    'Development::Management & Distribution',
    'Development::Other',
    'Development::Source Editors',
    'Education::Computer',
    'Education::Dictionaries',
    'Education::Geography',
    'Education::Kids',
    'Education::Languages',
    'Education::Mathematics',
    'Education::Other',
    'Education::Reference Tools',
    'Education::Science',
    'Education::Teaching & Training Tools',
    'Games & Entertainment::Action',
    'Games & Entertainment::Adventure & Roleplay',
    'Games & Entertainment::Arcade',
    'Games & Entertainment::Board',
    'Games & Entertainment::Card',
    'Games & Entertainment::Casino & Gambling',
    'Games & Entertainment::Kids',
    'Games & Entertainment::Online Gaming',
    'Games & Entertainment::Other',
    'Games & Entertainment::Puzzle & Word Games',
    'Games & Entertainment::Simulation',
    'Games & Entertainment::Sports',
    'Games & Entertainment::Strategy & War Games',
    'Games & Entertainment::Tools & Editors',
    'Graphic Apps::Animation Tools',
    'Graphic Apps::CAD',
    'Graphic Apps::Converters & Optimizers',
    'Graphic Apps::Editors',
    'Graphic Apps::Font Tools',
    'Graphic Apps::Gallery & Cataloging Tools',
    'Graphic Apps::Icon Tools',
    'Graphic Apps::Other',
    'Graphic Apps::Screen Capture',
    'Graphic Apps::Viewers',
    'Home & Hobby::Astrology/Biorhythms/Mystic',
    'Home & Hobby::Astronomy',
    'Home & Hobby::Cataloging',
    'Home & Hobby::Food & Drink',
    'Home & Hobby::Genealogy',
    'Home & Hobby::Health & Nutrition',
    'Home & Hobby::Other',
    'Home & Hobby::Personal Finance',
    'Home & Hobby::Personal Interest',
    'Home & Hobby::Recreation',
    'Home & Hobby::Religion',
    'Network & Internet::Ad Blockers',
    'Network & Internet::Browser Tools',
    'Network & Internet::Browsers',
    'Network & Internet::Download Managers',
    'Network & Internet::File Sharing/Peer to Peer',
    'Network & Internet::FTP Clients',
    'Network & Internet::Network Monitoring',
    'Network & Internet::Other',
    'Network & Internet::Remote Computing',
    'Network & Internet::Search/Lookup Tools',
    'Network & Internet::Terminal & Telnet Clients',
    'Network & Internet::Timers & Time Synch',
    'Network & Internet::Trace & Ping Tools',
    'Security & Privacy::Access Control',
    'Security & Privacy::Anti-Spam & Anti-Spy Tools',
    'Security & Privacy::Anti-Virus Tools',
    'Security & Privacy::Covert Surveillance',
    'Security & Privacy::Encryption Tools',
    'Security & Privacy::Other',
    'Security & Privacy::Password Managers',
    'Servers::Firewall & Proxy Servers',
    'Servers::FTP Servers',
    'Servers::Mail Servers',
    'Servers::News Servers',
    'Servers::Other Server Applications',
    'Servers::Telnet Servers',
    'Servers::Web Servers',
    'System Utilities::Automation Tools',
    'System Utilities::Backup & Restore',
    'System Utilities::Benchmarking',
    'System Utilities::Clipboard Tools',
    'System Utilities::File & Disk Management',
    'System Utilities::File Compression',
    'System Utilities::Launchers & Task Managers',
    'System Utilities::Other',
    'System Utilities::Printer',
    'System Utilities::Registry Tools',
    'System Utilities::Shell Tools',
    'System Utilities::System Maintenance',
    'System Utilities::Text/Document Editors',
    'Web Development::ASP & PHP',
    'Web Development::E-Commerce',
    'Web Development::Flash Tools',
    'Web Development::HTML Tools',
    'Web Development::Java & JavaScript',
    'Web Development::Log Analysers',
    'Web Development::Other',
    'Web Development::Site Administration',
    'Web Development::Wizards & Components',
    'Web Development::XML/CSS Tools',
  );
  static $categories;
  return _padfile_reindex($cats, $categories);
}

function _padfile_status_array() {
  static $stats = array(
    'Major Update',
    'Minor Update',
    'New Release',
    'Beta',
    'Alpha',
    'Media Only',
  );
  static $status;
  return _padfile_reindex($stats, $status);
}

function _padfile_license_type_array() {
  static $lic = array(
    'Shareware',
    'Freeware',
    'Adware',
    'Demo',
    'Commercial',
    'Data Only'
  );
  static $license;
  return _padfile_reindex($lic, $license);
}

function _padfile_install_support_array() {
  static $is = array(
    'Install and Uninstall',
    'Install Only',
    'No Install Support',
    'Uninstall Only',
  );
  static $install_support;
  return _padfile_reindex($is, $install_support);
}

function _padfile_expiration_base_array() {
  static $eb = array(
    'Days',
    'Uses',
    'Either/Or',
  );
  static $expiration_base;
  return _padfile_reindex($eb, $expiration_base);
}

function _padfile_operating_systems_array() {
  static $os = array(
    // This is from the previous version of PADFile (3.02)
    //'Win 3.1x',
    //'Win95',
    //'Win98',
    //'WinME',
    //'WinNT 3.x',
    //'WinNT 4.x',
    //'WinXP',
    //'Windows2000',
    //'Windows2003',
    //'Windows Tablet PC Edition 2005',
    //'Windows Media Center Edition 2005',
    //'Windows Vista Starter',
    //'Windows Vista Home Basic',
    //'Windows Vista Home Premium',
    //'Windows Vista Business',
    //'Windows Vista Enterprise',
    //'Windows Vista Ultimate',
    //'Windows Vista Home Basic x64',
    //'Windows Vista Home Premium x64',
    //'Windows Vista Business x64',
    //'Windows Vista Enterprise x64',
    //'Windows Vista Ultimate x64',
    //'Windows CE',
    //'Windows Mobile 2003',
    //'Windows Mobile 2005',
    //'Unix',
    //'Linux',
    //'Linux Console',
    //'Linux Gnome',
    //'Linux GPL',
    //'Linux Open Source',
    //'MS-DOS',
    //'AS/400',
    //'OS/2',
    //'OS/2 Warp',
    //'OS/2 Warp 4',
    //'MAC 68k',
    //'Mac PPC',
    //'Mac OS 9',
    //'Mac OS X',
    //'Mac OS X 10.1',
    //'Mac OS X 10.2',
    //'Mac OS X 10.3',
    //'Mac OS X 10.4',
    //'Mac OS X 10.5',
    //'Mac Other',
    //'Palm OS 1.0',
    //'Palm OS 2.0',
    //'Palm OS 2.1',
    //'Palm OS 3.0',
    //'Palm OS 3.1',
    //'Palm OS 3.2',
    //'Palm OS 4.0',
    //'Palm OS 5.0',
    //'Palm OS 6.0',
    //'Pocket PC',
    //'RIM',
    //'Symbian',
    //'OpenVMS',
    //'Netware',
    //'BEOS',
    //'Java',
    //'Android',
    //'Handheld/Mobile Other',
    //'Not Applicable',

    // List of OSes from PADFile 3.10
    'Android',
    'BlackBerry',
    'Handheld/Mobile Other',
    'iPhone',
    'iPod',
    'iTouch',
    'Java',
    'Linux',
    'Linux Console',
    'Linux Gnome',
    'Linux GPL',
    'Linux Open Source',
    'Mac OS X',
    'Mac Other',
    'MS-DOS',
    'Netware',
    'OpenVMS',
    'Palm',
    'Pocket PC',
    'Symbian',
    'Unix',
    'Win2000',
    'Win7 x32',
    'Win7 x64',
    'Win98',
    'WinMobile',
    'WinOther',
    'WinServer',
    'WinVista',
    'WinVista x64',
    'WinXP',
    'Other',
    'Not Applicable'
  );
  static $operating_systems;
  return _padfile_reindex($os, $operating_systems);
}

function _padfile_languages_array() {
  static $languages;
  if (!$languages) {
    require_once('./' . drupal_get_path('module', 'padfile') . '/padfile.lang.inc');
    $lang = _padfile_languages();
    _padfile_reindex($lang, $languages);
  }
  return $languages;
}

/**
 * Menu callback
 */
function padfile_content_list() {
  $form['#tree'] = TRUE;

  $has_page = module_exists('padfile_page');
  $padfiles = _padfile_get_padfiles();
  foreach ($padfiles as $pad) {
    $form[$pad->padid]['#pad'] = (array) $pad;
    $form[$pad->padid]['padid'] = array(
      '#value' => check_plain($pad->padid),
    );
    // TODO: we need a more complete status (i.e. Valid, Error #x,y,z, Off)
    $form[$pad->padid]['active'] = array(
      '#value' => $pad->active ? 'Active' : 'Off',
    );
    $form[$pad->padid]['program_name'] = array(
      '#value' => check_plain($pad->program_name),
    );
    $form[$pad->padid]['padfile_name'] = array(
      '#value' => check_plain($pad->padfile_name),
    );
    $form[$pad->padid]['version'] = array(
      '#value' => check_plain($pad->version),
    );
    $form[$pad->padid]['download_counter'] = array(
      '#value' => $pad->download_counter,
    );
    if ($pad->active) {
      $form[$pad->padid]['view'] = array(
        '#value' => l(t('view'), "padfile/{$pad->padfile_name}.xml", array('attributes' => array('target' => '_blank'))),
      );
      $form[$pad->padid]['export'] = array(
        '#value' => l(t('export'), "padfile/export/all/{$pad->padfile_name}.xml"),
      );
      // should we use some callback on this one?
      if ($has_page) {
        $form[$pad->padid]['page'] = array(
          '#value' => l(t('page'), "padpage/{$pad->padfile_name}"),
        );
      }
    }
    else {
      $form[$pad->padid]['view'] = array(
        '#value' => '&mdash;',
      );
      $form[$pad->padid]['export'] = array(
        '#value' => '&mdash;',
      );
      // should we use some callback on this one?
      if ($has_page) {
        $form[$pad->padid]['page'] = array(
          '#value' => '&mdash;',
        );
      }
    }
    $form[$pad->padid]['edit'] = array(
      '#value' => l(t('edit'), "admin/content/padfile/edit/$pad->padid"),
    );
    $form[$pad->padid]['delete'] = array(
      '#value' => l(t('delete'), "admin/content/padfile/delete/$pad->padid"),
    );
  }

  return $form;
}

/**
 * Menu callback (add a PAD file to the site)
 */
function padfile_form_padfile(&$form_state, $edit = array()) {
  global $user;

  // make sure we have proper defaults
  $edit += array(
    'uid' => $user->uid,
    'active' => 1,
    'download_counter' => 0,
    'link_nodes' => '',
    'company_name' => '',
    'address_1' => '',
    'address_2' => '',
    'city_town' => '',
    'state' => '',
    'zip' => '',
    'country' => '',
    'website' => 'http://',
    'author_first_name' => '',
    'author_last_name' => '',
    'author_email' => '',
    'contact_first_name' => '',
    'contact_last_name' => '',
    'contact_email' => '',
    'sales_email' => '',
    'support_email' => '',
    'general_email' => '',
    'sales_phone' => '',
    'support_phone' => '',
    'general_phone' => '',
    'fax_phone' => '',
    'padfile_name' => '',
    'program_name' => '',
    'version' => '',
    'release_date' => $_SERVER['REQUEST_TIME'],
    'currency' => 'USD',
    'cost' => '0.00',
    'type' => 'Freeware',
    'status' => 'Alpha',
    'install_support' => 'No Install Support',
    'category' => '',
    'operating_systems' => '',
    'requirements' => '',
    'languages' => '',
    'byte_size' => '',
    'change_log' => '',
    'info_url' => 'http://',
    'ordering_url' => 'http://',
    'screenshot_url' => 'http://',
    'icon_url' => 'http://',
    'download_url_primary' => 'http://',
    'download_url_secondary' => 'http://',
    'download_url_other1' => 'http://',
    'download_url_other2' => 'http://',
    'expiration_base' => 'Days',
    'expiration_count' => 0,
    'expiration_date' => '',
    'expiration_info' => '',
    'distribution_permissions' => '',
    'eula' => '',
    'module' => 'padfile',
  );

  // to use with checkboxes we need arrays
  $edit['operating_systems'] = explode(',', $edit['operating_systems']);
  $edit['languages'] = explode(',', $edit['languages']);

  if ($edit['release_date']) {
    $edit['release_date'] = gmdate('m/d/Y', $edit['release_date']);
  }
  if ($edit['expiration_date']) {
    $edit['expiration_date'] = gmdate('m/d/Y', $edit['expiration_date']);
  }
  else {
    // by default, it is 0, not an empty string
    $edit['expiration_date'] = '';
  }

  $form['status'] = array(
    '#type' => 'fieldset',
    '#title' => t('Status'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#description' => t('Current status of your PAD file.'),
  );
  $form['status']['active'] = array(
    '#type' => 'checkbox',
    '#title' => t('Active'),
    '#default_value' => $edit['active'],
    '#description' => t('Uncheck to keep this PAD file but hide it from external users.'),
    '#required' => TRUE,
  );
  $form['status']['reset_download'] = array(
    '#type' => 'checkbox',
    '#title' => t('Reset download'),
    '#default_value' => 0, // always zero!
    '#description' => t('Check to reset the download counter to the value specified below.'),
  );
  $form['status']['download_counter'] = array(
    '#type' => 'textfield',
    '#maxlength' => 10,
    '#size' => 10,
    '#title' => t('Download counter'),
    '#default_value' => $edit['download_counter'],
    '#description' => t('The new value for the download counter. Ignored if the Reset Download checkmark is not set.'),
  );
  $form['status']['link_nodes'] = array(
    '#type' => 'textarea',
    '#title' => t('Link on nodes'),
    '#default_value' => $edit['link_nodes'],
    '#description' => t('Enter a list of node names (or part of the name,) one per line, where a link to this padfile will be added.'),
    '#wysiwyg' => FALSE,
  );

  $form['company'] = array(
    '#type' => 'fieldset',
    '#title' => t('Owner (company or individual)'),
    '#collapsible' => TRUE,
    '#description' => t('Detailed information the company or author who owns this product.'),
  );
  $form['company']['company_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Company or author name'),
    '#maxlength' => 40,
    '#size' => 40,
    '#default_value' => $edit['company_name'],
    '#description' => t('Name of the company that owns this product. If no company, name of the author.'),
    '#required' => TRUE,
  );
  $form['company']['address_1'] = array(
    '#type' => 'textfield',
    '#title' => t('Address (line 1)'),
    '#maxlength' => 40,
    '#size' => 40,
    '#default_value' => $edit['address_1'],
  );
  $form['company']['address_2'] = array(
    '#type' => 'textfield',
    '#title' => t('Address (line 2)'),
    '#maxlength' => 40,
    '#size' => 40,
    '#default_value' => $edit['address_2'],
  );
  $form['company']['city_town'] = array(
    '#type' => 'textfield',
    '#title' => t('City or town name'),
    '#maxlength' => 40,
    '#size' => 40,
    '#default_value' => $edit['city_town'],
  );
  $form['company']['state'] = array(
    '#type' => 'textfield',
    '#title' => t('State'),
    '#maxlength' => 30,
    '#size' => 30,
    '#default_value' => $edit['state'],
  );
  $form['company']['zip'] = array(
    '#type' => 'textfield',
    '#title' => t('Zip or Postal Code'),
    '#maxlength' => 20,
    '#size' => 20,
    '#default_value' => $edit['zip'],
  );
  $form['company']['country'] = array(
    '#type' => 'textfield',
    '#title' => t('Country'),
    '#maxlength' => 40,
    '#size' => 40,
    '#default_value' => $edit['country'],
  );
  $form['company']['website'] = array(
    '#type' => 'textfield',
    '#title' => t('Website'),
    '#maxlength' => 80,
    '#default_value' => $edit['website'],
    '#description' => t('Website of the company or author. See Info URL below for the product URL.'),
  );

  $form['individuals'] = array(
    '#type' => 'fieldset',
    '#title' => t('Individuals details'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#description' => t('Detailed information about different people one can contact in regard to this product.'),
  );
  $form['individuals']['author_first_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Author first name'),
    '#maxlength' => 30,
    '#size' => 30,
    '#default_value' => $edit['author_first_name'],
  );
  $form['individuals']['author_last_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Author last name'),
    '#maxlength' => 30,
    '#size' => 30,
    '#default_value' => $edit['author_last_name'],
  );
  $form['individuals']['author_email'] = array(
    '#type' => 'textfield',
    '#title' => t('Author email'),
    '#maxlength' => 80,
    '#default_value' => $edit['author_email'],
    '#description' => t('i.e. joe.smith@example.com, WARNING: This email will be made public via the PAD file.'),
  );
  $form['individuals']['contact_first_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Contact first name'),
    '#maxlength' => 30,
    '#size' => 30,
    '#default_value' => $edit['contact_first_name'],
  );
  $form['individuals']['contact_last_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Contact last name'),
    '#maxlength' => 30,
    '#size' => 30,
    '#default_value' => $edit['contact_last_name'],
  );
  $form['individuals']['contact_email'] = array(
    '#type' => 'textfield',
    '#title' => t('Contact Email'),
    '#maxlength' => 80,
    '#default_value' => $edit['contact_email'],
    '#description' => t('i.e. product.name@example.com, WARNING: This email will be made public via the PAD file.'),
  );

  $form['contact'] = array(
    '#type' => 'fieldset',
    '#title' => t('Commercial contacts'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#description' => t('Detailed contact information for this product.'),
  );
  $form['contact']['sales_email'] = array(
    '#type' => 'textfield',
    '#title' => t('Sales email'),
    '#maxlength' => 80,
    '#default_value' => $edit['sales_email'],
    '#description' => t('i.e. sales@example.com, WARNING: This email will be made public via the PAD file.'),
  );
  $form['contact']['support_email'] = array(
    '#type' => 'textfield',
    '#title' => t('Support email'),
    '#maxlength' => 80,
    '#default_value' => $edit['support_email'],
    '#description' => t('i.e. support@example.com, WARNING: This email will be made public via the PAD file.'),
  );
  $form['contact']['general_email'] = array(
    '#type' => 'textfield',
    '#title' => t('General email'),
    '#maxlength' => 80,
    '#default_value' => $edit['general_email'],
    '#description' => t('i.e. contact@example.com, WARNING: This email will be made public via the PAD file.'),
  );
  $form['contact']['sales_phone'] = array(
    '#type' => 'textfield',
    '#title' => t('Sales phone'),
    '#maxlength' => 40,
    '#size' => 40,
    '#default_value' => $edit['sales_phone'],
    '#description' => t('Please, include the country code.'),
  );
  $form['contact']['support_phone'] = array(
    '#type' => 'textfield',
    '#title' => t('Support phone'),
    '#maxlength' => 40,
    '#size' => 40,
    '#default_value' => $edit['support_phone'],
    '#description' => t('Please, include the country code.'),
  );
  $form['contact']['general_phone'] = array(
    '#type' => 'textfield',
    '#title' => t('General phone'),
    '#maxlength' => 40,
    '#size' => 40,
    '#default_value' => $edit['general_phone'],
    '#description' => t('Please, include the country code.'),
  );
  $form['contact']['fax_phone'] = array(
    '#type' => 'textfield',
    '#title' => t('Fax number'),
    '#maxlength' => 40,
    '#size' => 40,
    '#default_value' => $edit['fax_phone'],
    '#description' => t('Please, include the country code.'),
  );

  $form['program'] = array(
    '#type' => 'fieldset',
    '#title' => t('Program information'),
    '#collapsible' => TRUE,
    '#description' => t('Detailed information about this software product.'),
  );
  $form['program']['padfile_name'] = array(
    '#type' => 'textfield',
    '#title' => t('PAD File name'),
    '#maxlength' => 80,
    '#size' => 40,
    '#default_value' => $edit['padfile_name'],
    '#description' => t('Name of the XML PAD File without the extension (usually all lowercase, no space, alphanumeric with underscores.)'),
    '#required' => TRUE,
  );
  $form['program']['program_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Program name'),
    '#maxlength' => 40,
    '#size' => 40,
    '#default_value' => $edit['program_name'],
    '#description' => t('Public name of your software product. Do not include version information in the name.'),
    '#required' => TRUE,
  );
  $form['program']['version'] = array(
    '#type' => 'textfield',
    '#title' => t('Program version'),
    '#maxlength' => 15,
    '#size' => 15,
    '#default_value' => $edit['version'],
    '#description' => t('The version information. Can be any alphanumeric characters, period (.), underscore (_) and dash (-).'),
  );
  $form['program']['byte_size'] = array(
    '#type' => 'textfield',
    '#title' => t('Size of the file in bytes'),
    '#maxlength' => 16,
    '#size' => 15,
    '#default_value' => $edit['byte_size'],
    '#description' => t('The exact size in bytes of your download file.'),
    '#required' => TRUE,
  );
  $form['program']['release_date'] = array(
    '#type' => 'textfield',
    '#title' => t('Release date'),
    '#maxlength' => 15,
    '#size' => 15,
    '#default_value' => $edit['release_date'],
    '#description' => t('Enter the date when this product was released (MM/DD/YYYY)'),
    '#required' => TRUE,
  );
  $form['program']['status'] = array(
    '#type' => 'select',
    '#title' => t('Development status'),
    '#default_value' => $edit['status'],
    '#options' => _padfile_status_array(),
    '#description' => t('Select the current status of the program.'),
  );
  $form['program']['change_log'] = array(
    '#type' => 'textarea',
    '#title' => t('Changes since last release'),
    '#maxlength' => 300,
    '#rows' => 5,
    '#default_value' => $edit['change_log'],
    '#description' => t('Information about the changes you made since your last release. (Limit: 300 characters)'),
    '#wysiwyg' => FALSE,
  );
  require_once('./' . drupal_get_path('module', 'padfile') . '/padfile.currencies.inc');
  $form['program']['currency'] = array(
    '#type' => 'select',
    '#title' => t('Currency'),
    '#default_value' => $edit['currency'],
    '#options' => _padfile_currencies_array(),
    '#description' => t('Select the currency used to define the cost of your program.'),
  );
  $form['program']['cost'] = array(
    '#type' => 'textfield',
    '#title' => t('Cost'),
    '#maxlength' => 15,
    '#size' => 15,
    '#default_value' => sprintf("%.02f", $edit['cost']),
    '#description' => t('Enter a decimal number with 2 decimals representing the price of your software. Do not include any currency sign (such as $, £ or ¥). If your cost is not 0.00, then you must also change the Type of license for your program (see the License box).'),
  );

  $form['category'] = array(
    '#type' => 'fieldset',
    '#title' => t('Categorization'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#description' => t('Detailed information to categorize your software product.'),
  );
  $form['category']['install_support'] = array(
    '#type' => 'select',
    '#title' => t('Development status'),
    '#default_value' => $edit['install_support'],
    '#options' => _padfile_install_support_array(),
    '#description' => t('Select the available installation mode(s).'),
  );
  $form['category']['category'] = array(
    '#type' => 'select',
    '#title' => t('Select the one category that best matches your program.'),
    '#default_value' => $edit['category'],
    '#options' => _padfile_categories_array(),
    '#description' => t('How many days or how many times (see Expiration base) the software can be used.'),
  );
  $form['category']['os'] = array(
    '#type' => 'fieldset',
    '#title' => t('Operating systems'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#description' => t('All the operating systems on which the program functions.'),
  );
  $form['category']['os']['operating_systems'] = array(
    '#type' => 'checkboxes',
    '#default_value' => $edit['operating_systems'],
    '#options' => _padfile_operating_systems_array(),
    '#description' => t('Select all that apply or "Not Applicable" if it does not matter (i.e. music, picture, video).'),
  );
  $form['category']['requirements'] = array(
    '#type' => 'textarea',
    '#title' => t('Requirements'),
    '#maxlength' => 100,
    '#rows' => 5,
    '#default_value' => $edit['requirements'],
    '#description' => t('Additional requirements for the software to run. (Limit: 100 characters.)'),
    '#wysiwyg' => FALSE,
  );
  $form['category']['language_field'] = array(
    '#type' => 'fieldset',
    '#title' => t('Languages'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#description' => t('All the languages offered by your program.'),
  );
  $form['category']['language_field']['languages'] = array(
    '#type' => 'checkboxes',
    '#default_value' => $edit['languages'],
    '#options' => _padfile_languages_array(),
    '#description' => t('Select all that apply and "Other" if you cannot find all the proper language(s).'),
  );

  $form['urls'] = array(
    '#type' => 'fieldset',
    '#title' => t('Descriptions/Pictures'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#description' => t('URLs to useful data in regard to the program.'),
  );
  $form['urls']['info_url'] = array(
    '#type' => 'textfield',
    '#maxlength' => 130,
    '#title' => t('Program URL'),
    '#default_value' => $edit['info_url'],
    '#description' => t('This URL must point to the main page describing your program in details.'),
  );
  $form['urls']['ordering_url'] = array(
    '#type' => 'textfield',
    '#maxlength' => 130,
    '#title' => t('Ordering URL'),
    '#default_value' => $edit['ordering_url'],
    '#description' => t('URL where customers can purchase your program.'),
  );
  $form['urls']['screenshot_url'] = array(
    '#type' => 'textfield',
    '#rows' => 130,
    '#title' => t('Screenshot URL'),
    '#default_value' => $edit['screenshot_url'],
    '#description' => t('URL to a screenshot of your program (must be a PNG, JPG or GIF).'),
  );
  $form['urls']['icon_url'] = array(
    '#type' => 'textfield',
    '#rows' => 130,
    '#title' => t('Icon URL'),
    '#default_value' => $edit['icon_url'],
    '#description' => t('URL to your program icon (must be a PNG, JPG or GIF of 32x32 pixels exactly).'),
  );

  $form['download'] = array(
    '#type' => 'fieldset',
    '#title' => t('Download'),
    '#collapsible' => TRUE,
    '#description' => t('URLs to download your program.'),
  );
  $form['download']['download_url_primary'] = array(
    '#type' => 'textfield',
    '#maxlength' => 130,
    '#title' => t('Primary URL'),
    '#default_value' => $edit['download_url_primary'],
    '#description' => t('The main URL where customers can download your software.'),
    '#required' => TRUE,
  );
  $form['download']['backup'] = array(
    '#type' => 'fieldset',
    '#title' => t('Backup URLs'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#description' => t('Extra URLs one can use to download your program.'),
  );
  $form['download']['backup']['download_url_secondary'] = array(
    '#type' => 'textfield',
    '#maxlength' => 130,
    '#title' => t('Backup URL 1'),
    '#default_value' => $edit['download_url_secondary'],
  );
  $form['download']['backup']['download_url_other1'] = array(
    '#type' => 'textfield',
    '#maxlength' => 130,
    '#title' => t('Backup URL 2'),
    '#default_value' => $edit['download_url_other1'],
  );
  $form['download']['backup']['download_url_other2'] = array(
    '#type' => 'textfield',
    '#maxlength' => 130,
    '#title' => t('Backup URL 3'),
    '#default_value' => $edit['download_url_other2'],
  );

  $form['expiration'] = array(
    '#type' => 'fieldset',
    '#title' => t('Expiration'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#description' => t('Information about the demonstration version expiration scheme.'),
  );
  $form['expiration']['expiration_base'] = array(
    '#type' => 'select',
    '#title' => t('Select what the expiration count represents'),
    '#default_value' => $edit['expiration_base'],
    '#options' => _padfile_expiration_base_array(),
    '#description' => t('Select what "Expiration count" represents.'),
  );
  $form['expiration']['expiration_count'] = array(
    '#type' => 'textfield',
    '#title' => t('Number of days or times one can use the software'),
    '#maxlength' => 15,
    '#size' => 15,
    '#default_value' => $edit['expiration_count'],
    '#description' => t('How many days or how many times (see Expiration base) the software can be used.'),
  );
  $form['expiration']['expiration_date'] = array(
    '#type' => 'textfield',
    '#title' => t('Date when the program expires'),
    '#maxlength' => 15,
    '#size' => 15,
    '#default_value' => $edit['expiration_date'],
    '#description' => t('Date when the product expires (MM/DD/YYYY).'),
  );
  $form['expiration']['expiration_info'] = array(
    '#type' => 'textarea',
    '#maxlength' => 100,
    '#rows' => 5,
    '#title' => t('Describe the expiration scheme'),
    '#default_value' => $edit['expiration_info'],
    '#description' => t('Select what the "Expiration count" represents. (limit: 100 characters)'),
    '#wysiwyg' => FALSE,
  );

  $form['license'] = array(
    '#type' => 'fieldset',
    '#title' => t('License'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#description' => t('Licensing information about your program.'),
  );
  $form['license']['type'] = array(
    '#type' => 'select',
    '#title' => t('Type of license'),
    '#default_value' => $edit['type'],
    '#options' => _padfile_license_type_array(),
    '#description' => t('Select the type of license. Only the offered types are valid. Note that Freeware implies that the cost be 0.00.'),
  );
  $form['license']['distribution_permissions'] = array(
    '#type' => 'textarea',
    '#maxlength' => 2000,
    '#rows' => 10,
    '#title' => t('Distribution permissions'),
    '#default_value' => $edit['distribution_permissions'],
    '#description' => t('Description of your policy in regard to re-distributing the software. (maximum 2,000 characters)'),
    '#wysiwyg' => FALSE,
  );
  $form['license']['eula'] = array(
    '#type' => 'textarea',
    '#maxlength' => 20000,
    '#rows' => 25,
    '#title' => t('EULA'),
    '#default_value' => $edit['eula'],
    '#description' => t('The End User License Agreement for this program.'),
    '#wysiwyg' => FALSE,
  );

  $form['modified_on'] = array(
    '#type' => 'value',
    '#value' => $_SERVER['REQUEST_TIME'],
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
    '#weight' => 10,
  );
  if (isset($edit['padid'])) {
    $form['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Delete'),
      '#weight' => 11,
    );
    $form['padid'] = array(
      '#type' => 'value',
      '#value' => $edit['padid'],
    );
  }
  else {
    // this one is set only when you add to the database
    $form['created_on'] = array(
      '#type' => 'value',
      '#value' => $_SERVER['REQUEST_TIME'],
    );
  }
  $form['module'] = array(
    '#type' => 'value',
    '#value' => $edit['module'],
  );
  $form['uid'] = array(
    '#type' => 'value',
    '#value' => $edit['uid'],
  );

  return $form;
}

/**
 * Menu callback (Confirm before delete)
 */
function padfile_confirm_delete(&$form_state, $padfile) {
  $form['type'] = array(
    '#type' => 'value',
    '#value' => 'padfile',
  );
  $form['name'] = array(
    '#type' => 'value',
    '#value' => $padfile->program_name,
  );
  $form['padid'] = array(
    '#type' => 'value',
    '#value' => $padfile->padid,
  );
  $form['delete'] = array(
    '#type' => 'value',
    '#value' => TRUE,
  );
  return confirm_form(
    $form,
    t('Are you sure you want to delete the PAD File named "@name"?',
        array('@name' => $padfile->program_name)),
    'admin/content/padfile',
    t('Deleting a PAD File will delete all data associated with the PAD File. This action cannot be undone. '
     .'If you just want to hide the PAD File for a while, !edit it and change its Status by unchecking "Valid".',
        array('!edit' => l('edit', 'admin/content/padfile/edit/' . $padfile->padid))) . '<br/><br/>',
    t('Delete'),
    t('Cancel')
  );
}

/**
 * Menu callback (Edit/Delete a PAD File)
 */
function padfile_content_edit($padfile) {
  if ((isset($_POST['op']) && $_POST['op'] == t('Delete')) || isset($_POST['confirm'])) {
    return drupal_get_form('padfile_confirm_delete', $padfile);
  }
  return drupal_get_form('padfile_form_padfile', (array)$padfile);
}

/**
 * Used confirmed he wanted to delete the PAD File
 */
function padfile_confirm_delete_submit($form, &$form_state) {
  padfile_delete($form_state['values']['padid']);
  drupal_set_message(t('Deleted PAD File @name.', array('@name' => $form_state['values']['name'])));
  watchdog('padfile', 'Deleted PAD File @name.', array('@name' => $form_state['values']['name']), WATCHDOG_NOTICE);
  $form_state['redirect'] = 'admin/content/padfile';
}

/**
 * hook_validate()
 */
function padfile_form_padfile_validate($form, &$form_state) {
  // CHECK URLs
  // Note that the URLs marked as required will be checked by the system
  // BEFORE we have a chance to clean the form state. So we have to check
  // them again here
  _padfile_clean_urls($form_state);
  if (!$form_state['values']['download_url_primary']) {
    form_set_error('download_url_primary', t('The primary URL was not defined.'));
  }

  // CHECK DATES
  if ($form_state['values']['release_date']) {
    $rd = explode('/', $form_state['values']['release_date']);
    if (!checkdate($rd[0], $rd[1], $rd[2])) {
      form_set_error('release_date', t('The program release date is not valid.'));
    }
  }
  if ($form_state['values']['expiration_date']) {
    $ed = explode('/', $form_state['values']['expiration_date']);
    if (!checkdate($ed[0], $ed[1], $ed[2])) {
      form_set_error('expiration_date', t('The expiration date is not valid.'));
    }
  }

  // CHECK NUMBERS
  if (!is_numeric($form_state['values']['byte_size'])) {
    form_set_error('byte_size', t('The size of your file must be a number.'));
  }
  elseif ($form_state['values']['byte_size'] < variable_get('padfile_minimum_byte_size', 0)) {
    form_set_error('byte_size', t('The program minimum file size is !min, the program size is currently set to !size.',
      array('!min' => variable_get('padfile_minimum_byte_size', 0),
            '!size' => $form_state['values']['byte_size'])));
  }
  if ($form_state['values']['download_counter']) {
    if (!is_numeric($form_state['values']['download_counter'])) {
      form_set_error('download_counter', 'The Download counter needs to be set to a valid number.');
    }
    elseif ($form_state['values']['byte_size'] < 0) {
      form_set_error('download_counter', 'The Download counter cannot be negative.');
    }
  }
  if ($form_state['values']['expiration_count']) {
    if (!is_numeric($form_state['values']['expiration_count'])) {
      form_set_error('expiration_count', t('The Expiration count must be a positive number.'));
    }
    elseif ($form_state['values']['expiration_count'] < 0) {
      form_set_error('expiration_count', t('The Expiration count cannot be negative.'));
    }
  }
  if ($form_state['values']['cost']) {
    if (!is_numeric($form_state['values']['cost'])) {
      form_set_error('cost', t('The Count must be a positive number.'));
    }
    elseif ($form_state['values']['cost'] < 0) {
      form_set_error('cost', t('The Cost cannot be negative.'));
    }
    elseif ($form_state['values']['cost'] != 0 && $form_state['values']['type'] == 'Freeware') {
      form_set_error('type', t('The license Type cannot be Freeware with a cost that is not zero.'));
    }
  }

  // CHECK EMAILS
  if ($form_state['values']['author_email']) {
    if (strstr($form_state['values']['author_email'], '@') === FALSE) {
      form_set_error('author_email', 'The Author email does not look like a valid email address.');
    }
  }
  if ($form_state['values']['contact_email']) {
    if (strstr($form_state['values']['contact_email'], '@') === FALSE) {
      form_set_error('contact_email', 'The Author email does not look like a valid email address.');
    }
  }
  if ($form_state['values']['sales_email']) {
    if (strstr($form_state['values']['sales_email'], '@') === FALSE) {
      form_set_error('sales_email', 'The Author email does not look like a valid email address.');
    }
  }
  if ($form_state['values']['support_email']) {
    if (strstr($form_state['values']['support_email'], '@') === FALSE) {
      form_set_error('support_email', 'The Author email does not look like a valid email address.');
    }
  }
  if ($form_state['values']['general_email']) {
    if (strstr($form_state['values']['general_email'], '@') === FALSE) {
      form_set_error('general_email', 'The Author email does not look like a valid email address.');
    }
  }
}

/**
 * Remove http://, ftp://, etc. in uncompleted URLs.
 */
function _padfile_clean_urls(&$form_state) {
  static $urls = array(
    'website',
    'info_url',
    'ordering_url',
    'screenshot_url',
    'icon_url',
    'download_url_primary',
    'download_url_secondary',
    'download_url_other1',
    'download_url_other2',
  );
  // this changes depending on the URL defined
  // and it is just plainly stupid to not allow https where http is defined?!
  static $protocols = array(
    'http://',
    'https://',
    'ftp://',
  );

  // for all the URLs, if still empty, make it completely empty
  foreach ($urls as $u) {
    $url = $form_state['values'][$u];
    $lu = drupal_strlen($url);
    foreach ($protocols as $p) {
      $lp = drupal_strlen($p);
      if ($lu > $lp) {
        // not a match
        continue;
      }
      for ($i = 0; $i < $lu; ++$i) {
        if ($url[$i] != $p[$i]) {
          // not a match
          break;
        }
      }
      if ($i == $lu) {
        // it is a match, get rid of that entry
        // (do not unset() otherwise the previous value will stick!)
        $form_state['values'][$u] = '';
        break;
      }
    }
  }
}

/**
 * hook_submit()
 */
function padfile_form_padfile_submit($form, &$form_state) {
  global $user;

  if (!isset($form_state['module'])) {
    $form_state['module'] = 'padfile';
  }
  if ($form_state['values']['padid']) {
    $save_edit = 'padid';
  }
  else {
    $save_edit = NULL;
  }
  if (!$form_state['values']['uid']) {
    $form_state['values']['uid'] = $user->uid;
  }

  _padfile_clean_urls($form_state);

  // to save in the database we need strings instead of the checkboxes arrays
  // (and we do not want all the zeroes that we clear first)
  foreach ($form_state['values']['operating_systems'] as $k => $v) {
    if (!$form_state['values']['operating_systems'][$k]) {
      unset($form_state['values']['operating_systems'][$k]);
    }
  }
  $form_state['values']['operating_systems'] = implode(',', $form_state['values']['operating_systems']);

  foreach ($form_state['values']['languages'] as $k => $v) {
    if (!$form_state['values']['languages'][$k]) {
      unset($form_state['values']['languages'][$k]);
    }
  }
  $form_state['values']['languages'] = implode(',', $form_state['values']['languages']);

  // transform dates to Unix time
  if ($form_state['values']['release_date']) {
    $rd = explode('/', $form_state['values']['release_date']);
    $form_state['values']['release_date'] = gmmktime(0, 0, 0, $rd[0], $rd[1], $rd[2], FALSE);
  }
  if ($form_state['values']['expiration_date']) {
    $ed = explode('/', $form_state['values']['expiration_date']);
    $form_state['values']['expiration_date'] = gmmktime(0, 0, 0, $ed[0], $ed[1], $ed[2], FALSE);
  }

  if ($form_state['values']['padfile_name'] != preg_replace('/ /', '', $form_state['values']['padfile_name'])) {
    drupal_set_message('The PAD File filename includes characters that you should not use.', 'warning');
  }

  // if reset download is set, keep the counter, otherwise forget it
  if (!$form_state['values']['reset_download']) {
    // by doing and unset() we prevent a disaster (i.e. the counter in the database remains unchanged)
    unset($form_state['values']['download_counter']);
  }

  drupal_write_record('padfile', $form_state['values'], $save_edit);

  // Switch to the editor so people can now edit the descriptions
  // (instead of going back to the list of PAD Files)
  $form_state['padid'] = $form_state['values']['padid'];
  $form_state['redirect'] = 'admin/content/padfile/edit/' . $form_state['padid'];
}

/**
 * Menu callback (Offer a way to the user to import his PAD Files)
 */
function padfile_form_import(&$form_state) {
  // make sure we are sent the proper type of data
  $form['#attributes'] = array(
    'enctype' => "multipart/form-data"
  );

  $form['import'] = array(
    '#type' => 'fieldset',
    '#title' => t('Filename'),
    '#description' => t('Select the file you want to import.'),
  );
  $form['import']['xml_padfile'] = array(
    '#type' => 'file',
    '#title' => t('XML PAD File'),
    '#description' => t('Browse for a PAD File available on one of your storage devices.'),
    '#size' => 30,
    //'#required' => TRUE, -- not available
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );

  return $form;
}

function padfile_validate_is_xml(&$file) {
  if ($file->filemime == 'application/xml' || $file->filemime == 'text/xml') {
    return array();
  }

  return array(t('Only XML files are allowed. This file is of type !mime.', array('!mime' => $file->filemime)));
}

function padfile_form_import_validate($form, &$form_state) {
  $validators = array(
    'padfile_validate_is_xml' => array(),
  );
  if (!file_save_upload('xml_padfile', $validators)) {
    form_set_error('xml_padfile', 'A valid XML file is required for the import feature to work.');
  }
}

function padfile_form_import_submit($form, &$form_state) {
  global $user;

  $pad = array();

  $validators = array(
    'padfile_validate_is_xml' => array(),
  );
  $xml_padfile = file_save_upload('xml_padfile', $validators);
  $doc = new DOMDocument();
  if ($doc->load($xml_padfile->filepath) === FALSE) {
    drupal_set_message('The XML file could not be parsed. Please, make sure it is a valid PAD File.', 'error');
    return;
  }

  static $fields = array(
    'company_name' => 'Company_Name',
    'address_1' => 'Address_1',
    'address_2' => 'Address_2',
    'city_town' => 'City_Town',
    'state' => 'State_Province',
    'zip' => 'Zip_Postal_Code',
    'country' => 'Country',
    'website' => 'Company_WebSite_URL',
    'author_first_name' => 'Author_First_Name',
    'author_last_name' => 'Author_Last_Name',
    'author_email' => 'Author_Email',
    'contact_first_name' => 'Contact_First_Name',
    'contact_last_name' => 'Contact_Last_Name',
    'contact_email' => 'Contact_Email',
    'sales_email' => 'Sales_Email',
    'support_email' => 'Support_Email',
    'general_email' => 'General_Email',
    'sales_phone' => 'Sales_Phone',
    'support_phone' => 'Support_Phone',
    'general_phone' => 'General_Phone',
    'fax_phone' => 'Fax_Phone',
    'program_name' => 'Program_Name',
    'version' => 'Program_Version',
    'release_month' => 'Program_Release_Month',
    'release_day' => 'Program_Release_Day',
    'release_year' => 'Program_Release_Year',
    'cost_dollars' => 'Program_Cost_Dollars',
    'currency' => 'Program_Cost_Other_Code',
    'cost_other' => 'Program_Cost_Other',
    'type' => 'Program_Type',
    'status' => 'Program_Release_Status',
    'install_support' => 'Program_Install_Support',
    'operating_systems' => 'Program_OS_Support',
    'languages' => 'Program_Language',
    'byte_size' => 'File_Size_Bytes',
    'expiration_count' => 'Expire_Count',
    'expiration_base' => 'Expire_Based_On',
    'expiration_info' => 'Expire_Other_Info',
    'expiration_month' => 'Expire_Month',
    'expiration_day' => 'Expire_Day',
    'expiration_year' => 'Expire_Year',
    'change_log' => 'Program_Change_Info',
    'category' => 'Program_Category_Class',
    'requirements' => 'Program_System_Requirements',
    'info_url' => 'Application_Info_URL',
    'ordering_url' => 'Application_Order_URL',
    'screenshot_url' => 'Application_Screenshot_URL',
    'icon_url' => 'Application_Icon_URL',
    'padfile_fullname' => 'Application_XML_File_URL',
    'download_url_primary' => 'Primary_Download_URL',
    'download_url_secondary' => 'Secondary_Download_URL',
    'download_url_other1' => 'Additional_Download_URL_1',
    'download_url_other2' => 'Additional_Download_URL_2',
    'distribution_permissions' => 'Distribution_Permissions',
    'eula' => 'EULA',
  );

  // extract the data from the XML file and save in the $pad array
  foreach ($fields as $f => $tag) {
    $nodes = $doc->getElementsByTagName($tag);
    if ($nodes->length) {
      $item = $nodes->item(0);
      $pad[$f] = $item->textContent;
    }
  }

  if ($pad['release_month'] || $pad['release_day'] || $pad['release_year']) {
    $pad['release_date'] = gmmktime(0, 0, 0, $pad['release_month'], $pad['release_day'], $pad['release_year'], FALSE);
  }
  if ($pad['currency'] && $pad['cost_other']) {
    $pad['cost'] = $pad['cost_other'];
  }
  else {
    $pad['currency'] = 'USD';
    $pad['cost'] = $pad['cost_other'];
  }

  if ($pad['expiration_month'] || $pad['expiration_day'] || $pad['expiration_year']) {
    $pad['expiration_date'] = gmmktime(0, 0, 0, $pad['expiration_month'], $pad['expiration_day'], $pad['expiration_year'], FALSE);
  }

  $pad['padfile_name'] = basename($pad['padfile_fullname'], '.xml');

  $result = db_query("SELECT padid FROM {padfile} WHERE padfile_name = '%s'", $pad['padfile_name']);
  $existing_pad = db_fetch_object($result);
  if ($existing_pad) {
    drupal_set_message(t('PAD File @name already exists. Import cancelled.', array('@name' => $pad['padfile_name'])), 'error');
  }
  else {

    $pad['created_on'] = $_SERVER['REQUEST_TIME'];
    $pad['modified_on'] = $_SERVER['REQUEST_TIME'];
    $pad['uid'] = $user->uid;

    // save the main PAD File already, then work on descriptions
    drupal_write_record('padfile', $pad);

    $result = db_query("SELECT padid FROM {padfile} WHERE padfile_name = '%s'", $pad['padfile_name']);
    $new_pad = db_fetch_object($result);
    if (!$new_pad) {
      drupal_set_message(t("New PAD File @name not found. Descriptions will not be loaded.\n", array('@name' => $pad['padfile_name'])), 'error');
    }
    else {
      static $desc_fields = array(
        'keywords' => 'Keywords',
        'desc45' => 'Char_Desc_45',
        'desc80' => 'Char_Desc_80',
        'desc250' => 'Char_Desc_250',
        'desc450' => 'Char_Desc_450',
        'desc2000' => 'Char_Desc_2000',
      );

      $nodes = $doc->getElementsByTagName('Program_Descriptions');
      if ($nodes->length) {
        $program_desc = $nodes->item(0);

        $all_languages = _padfile_languages_array();
        for ($language = $program_desc->firstChild; $language; $language = $language->nextSibling) {
          if (!$language->tagName || !isset($all_languages[$language->tagName])) {
            // skip empty DOMText entries and invalid (unsupported) languages
            continue;
          }
          // extract the data from the XML file and save in the $desc array
          $desc = array();
          foreach ($desc_fields as $f => $tag) {
            $descriptions = $language->getElementsByTagName($tag);
            if ($descriptions->length) {
              $item = $descriptions->item(0);
              $desc[$f] = $item->textContent;
            }
          }
          $desc['padid'] = $new_pad->padid;
          $desc['language'] = $language->tagName;
          // save these descriptions
          drupal_write_record('padfile_descriptions', $desc);
        }
      }

    }
  }

  $form_state['redirect'] = 'admin/content/padfile/list';
}

/**
 * Menu callback (List all the descriptions available for a PAD File)
 */
function padfile_content_description_list(&$form_state, $padfile) {
  $form['#tree'] = TRUE;

  $descriptions = _padfile_get_descriptions($padfile->padid);
  foreach ($descriptions as $d) {
    $form[$d->language]['#description'] = (array) $d;
    $form[$d->language]['padid'] = array(
      '#value' => check_plain($pad->padid),
    );
    $form[$d->language]['language'] = array(
      '#value' => check_plain($d->language),
    );
    $form[$d->language]['desc45'] = array(
      '#value' => check_plain($d->desc45),
    );
    if ($padfile->active) {
      $form[$d->language]['view'] = array(
        '#value' => l(t('view'), "padfile/$d->language/{$padfile->padfile_name}.xml"),
      );
      $form[$d->language]['export'] = array(
        '#value' => l(t('export'), "padfile/export/$d->language/{$padfile->padfile_name}.xml"),
      );
      if (module_exists('padfile_page')) {
        $form[$d->language]['page'] = array(
          '#value' => l(t('page'), "padpage/{$padfile->padfile_name}"),
        );
      }
    }
    else {
      $form[$d->language]['view'] = array(
        '#value' => '&mdash;',
      );
      $form[$d->language]['export'] = array(
        '#value' => '&mdash;',
      );
      if (module_exists('padfile_page')) {
        $form[$d->language]['page'] = array(
          '#value' => '&mdash;',
        );
      }
    }
    $form[$d->language]['edit'] = array(
      '#value' => l(t('edit'), "admin/content/padfile/edit/$d->padid/desc_edit/$d->language"),
    );
    $form[$d->language]['delete'] = array(
      '#value' => l(t('delete'), "admin/content/padfile/edit/$d->padid/desc_delete/$d->language"),
    );
  }

  return $form;
}

/**
 * Retrieve a PAD File description
 *
 * \param[in] $padid     The identifier of the PAD File attached to the description to load
 * \param[in] $language  The language of the description to load
 *
 * \return An array composed of the raw rows from the database.
 */
function _padfile_descriptions_load($padid, $language) {

//var_dump($padid); var_dump($language);

  // temporary cache
  static $descriptions = array();

  $index = $padid . '/' . $language;

  if (!isset($descriptions[$index])) {
    // by default we mark it false, if it fails, it's not available anyway
    $padfiles[$index] = FALSE;

    // load the description
    $result = db_query("SELECT * FROM {padfile_descriptions} WHERE padid = %d AND language = '%s'", $padid, $language);
    if ($result) {
      $desc = db_fetch_object($result);
      if ($desc) {
        $descriptions[$index] = $desc;
      }
    }
    if (!$descriptions[$index]) {
      drupal_set_message(t('Description #@padid/@language does not exist.', array('@padid' => $padid, '@language' => $language)), 'error');
    }
  }

  return $descriptions[$index];
}

/**
 * Menu callback (Add/edit a description for a PAD File)
 */
function padfile_form_description(&$form_state, $padid, $language = NULL) {
  // TODO: should we check that $padid is valid?
  if ($language) {
    $editing = TRUE;
    $edit = _padfile_descriptions_load($padid, $language);
    if ($edit === FALSE) {
      $edit = array();
    }
    else {
      $edit = (array)$edit;
    }
  }
  else {
    $editing = FALSE;
    $edit = array();
  }

  // make sure we have proper defaults
  $edit += array(
    'padid' => $padid,
    'language' => 'English',
    'keywords' => '',
    'desc45' => '',
    'desc80' => '',
    'desc250' => '',
    'desc450' => '',
    'desc2000' => '',
    'module' => 'padfile',
  );

  $form['description'] = array(
    '#type' => 'fieldset',
    '#title' => t('Descriptions'),
    '#collapsible' => TRUE,
    '#description' => t('Language, keywords and different length descriptions for your PAD File.')
      . ($editing ? '<br/>' . t('Editing the !lang language descriptions.', array('!lang' => $edit['language'])) : ''),
  );
  if ($editing) {
    $form['language'] = array(
      '#type' => 'value',
      '#value' => $edit['language'],
    );
  }
  else {
    // read all about the languages already in use
    $in_use = _padfile_get_descriptions($padid);
    // get the complete list of languages available
    $languages = _padfile_languages_array();
    // remove from the complete list those that are already in use
    foreach ($in_use as $d) {
      unset($languages[$d->language]);
    }
    $form['description']['language'] = array(
      '#type' => 'select',
      '#default_value' => $edit['language'],
      '#options' => $languages,
      '#description' => t('Select the language of this description. You can have one set of description per language.'),
    );
  }
  $form['description']['keywords'] = array(
    '#type' => 'textarea',
    '#maxlength' => 250,
    '#rows' => 3,
    '#title' => t('Keywords'),
    '#default_value' => $edit['keywords'],
    '#description' => t('A set of keywords describing your product separated by commas. Avoid punctuation. (Limit: 250 characters)'),
    '#wysiwyg' => FALSE,
  );
  $form['description']['desc45'] = array(
    '#type' => 'textfield',
    '#title' => t('Short description'),
    '#maxlength' => 45,
    '#size' => 45,
    '#default_value' => $edit['desc45'],
    '#description' => t('A short description of the product in 45 characters.'),
    '#wysiwyg' => FALSE,
  );
  $form['description']['desc80'] = array(
    '#type' => 'textarea',
    '#title' => t('One line description'),
    '#maxlength' => 80,
    '#rows' => 2,
    '#default_value' => $edit['desc80'],
    '#description' => t('One line description of the product in 80 characters.'),
    '#wysiwyg' => FALSE,
  );
  $form['description']['desc250'] = array(
    '#type' => 'textarea',
    '#title' => t('Small description'),
    '#maxlength' => 250,
    '#rows' => 5,
    '#default_value' => $edit['desc250'],
    '#description' => t('A few lines of description of the product in 250 characters.'),
    '#wysiwyg' => FALSE,
  );
  $form['description']['desc450'] = array(
    '#type' => 'textarea',
    '#title' => t('Medium description'),
    '#maxlength' => 450,
    '#rows' => 7,
    '#default_value' => $edit['desc450'],
    '#description' => t('A description of the product in 450 characters.'),
    '#wysiwyg' => FALSE,
  );
  $form['description']['desc2000'] = array(
    '#type' => 'textarea',
    '#title' => t('Long description'),
    '#maxlength' => 2000,
    '#rows' => 10,
    '#default_value' => $edit['desc2000'],
    '#description' => t('A long description of the product in 2000 characters.'),
    '#wysiwyg' => FALSE,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  if ($editing) {
    $form['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Delete'),
    );
  }
  $form['padfile_editing'] = array(
    '#type' => 'value',
    '#value' => $editing ? 1 : 0,
  );
  $form['padid'] = array(
    '#type' => 'value',
    '#value' => $edit['padid'],
  );
  $form['module'] = array(
    '#type' => 'value',
    '#value' => $edit['module'],
  );

  return $form;
}

/**
 * Validation function to make sure that the descriptions don't include invalid
 * characters.
 */
function padfile_form_description_validate($form, &$form_state) {
  if (preg_match("/[\n<>]/", $form_state['values']['desc45'])) {
    form_set_error('desc45', t('The 45 characters description cannot include new-line, &lt; or &gt; characters.'));
  }
  if (preg_match("/[\n<>]/", $form_state['values']['desc80'])) {
    form_set_error('desc80', t('The 80 characters description cannot include new-line, &lt; or &gt; characters.'));
  }
  if (preg_match("/[\n<>]/", $form_state['values']['desc250'])) {
    form_set_error('desc250', t('The 250 characters description cannot include new-line, &lt; or &gt; characters.'));
  }
  if (preg_match("/[\n<>]/", $form_state['values']['desc450'])) {
    form_set_error('desc450', t('The 450 characters description cannot include new-line, &lt; or &gt; characters.'));
  }
}

/**
 * Submit function used to save the descriptions.
 */
function padfile_form_description_submit($form, &$form_state) {
  if (!isset($form_state['module'])) {
    $form_state['module'] = 'padfile';
  }
  if ($form_state['values']['padfile_editing']) {
    $save_edit = array('padid', 'language');
  }

  drupal_write_record('padfile_descriptions', $form_state['values'], $save_edit);

  // Go to the list of descriptions
  $form_state['padid'] = $form_state['values']['padid'];
  $form_state['redirect'] = 'admin/content/padfile/edit/' . $form_state['padid'] . '/desc_list';
}

/**
 * Menu callback (Edit a description)
 */
function padfile_content_description_edit($padid, $language) {
  if ((isset($_POST['op']) && $_POST['op'] == t('Delete')) || isset($_POST['confirm'])) {
    return drupal_get_form('padfile_confirm_description_delete', $padid, $language);
  }
  return drupal_get_form('padfile_form_description', $padid, $language);
}

/**
 * Menu callback (Delete a description--ask for confirmation)
 */
function padfile_confirm_description_delete(&$form_state, $padid, $language) {
  $form['type'] = array(
    '#type' => 'value',
    '#value' => 'padfile_descriptions',
  );
  $form['language'] = array(
    '#type' => 'value',
    '#value' => $language,
  );
  $form['padid'] = array(
    '#type' => 'value',
    '#value' => $padid,
  );
  $form['delete'] = array(
    '#type' => 'value',
    '#value' => TRUE,
  );
  return confirm_form(
    $form,
    t('Are you sure you want to delete the PAD File descriptions for language "!lang"?',
        array('!lang' => $language)),
    'admin/content/padfile/' . $padid . '/desc_list',
    t('Deleting a PAD File description will delete all the descriptions associated with that entry. This action cannot be undone.') . '<br/><br/>',
    t('Delete'),
    t('Cancel')
  );
}

/**
 * Used confirmed he wanted to delete the PAD File
 */
function padfile_confirm_description_delete_submit($form, &$form_state) {
  padfile_descriptions_delete($form_state['values']['padid'], $form_state['values']['language']);
  drupal_set_message(t('Deleted PAD File description for language @lang.', array('@lang' => $form_state['values']['language'])));
  $form_state['redirect'] = 'admin/content/padfile/edit/' . $form_state['values']['padid'] . '/desc_list';
}


/**
 * Menu callback
 */
function padfile_form_settings(&$form_state) {
  $form['padfiles'] = array(
    '#type' => 'fieldset',
    '#title' => t('PAD Files'),
    '#collapsible' => TRUE,
  );
  $form['padfiles']['padfile_minimum_byte_size'] = array(
    '#title' => 'Program minimum byte size',
    '#type' => 'textfield',
    '#maxlength' => 10,
    '#size' => 10,
    '#default_value' => variable_get('padfile_minimum_byte_size', 0),
    '#description' => t('Define the minimum size, in bytes, of any one of your PAD File. Default is 0. Note: this value does not apply to existing PAD Files until you edit them.'),
  );

  $form['padfiles']['padfile_include_padmap'] = array(
    '#title' => 'Include PADmap tag',
    '#type' => 'checkbox',
    '#default_value' => variable_get('padfile_include_padmap', 1),
    '#description' => t('Select this checkbox to include the PADmap tag. This tag points back to your list of padfiles (under /padfile) and is included in all your PAD files by default. See !padmap for more information.', array('!padmap' => l('PADmap', 'http://padmaps.org/', array('attributes' => array('target' => '_blank'))))),
  );

  $form['padfiles']['padfile_include_padring'] = array(
    '#title' => 'Include PADRING tag',
    '#type' => 'checkbox',
    '#default_value' => variable_get('padfile_include_padring', 0),
    '#description' => t('Select this checkbox if you want to include the PADRING tag to your PAD Files. IMPORTANT: The tag is NOT included if you have exactly ONE PAD File defined since it would otherwise be empty. The PADRING is supported by different websites but it has the drawback of including the full list of all your PAD Files in all the PAD Files. If you have many PAD Files, it is recommanded that you do not check this box. See !padring for more information.', array('!padring' => l('Pad Ring', 'http://www.padring.com/', array('attributes' => array('target' => '_blank'))))),
  );

  return system_settings_form($form);
}

/**
 * Menu callback
 */
function padfile_form_tools(&$form_state) {
  $form['cache'] = array(
    '#type' => 'fieldset',
    '#title' => t('Cache'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#description' => t(
        'The XML PAD Files are created once and then cached in files for very fast retrieval. '
      . 'The files are deleted as required whenever you change a PAD File. '
      . 'However, if there seems to be a problem, feel free to clear everything with this button.'
    ),
  );
  $form['cache']['clear_cache'] = array(
    '#type' => 'submit',
    '#value' => t('Clear the entire PAD File cache'),
  );

  $form['mass'] = array(
    '#type' => 'fieldset',
    '#title' => t('Mass Action'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#description' => t(
        'The following buttons apply to all your PAD Files. '
      . '<span color="red">Be very careful as editing each PAD File can be tedious.</span> '
    ),
  );
  $form['mass']['desactivate_all'] = array(
    '#type' => 'submit',
    '#value' => t('Desactivate all the PAD File'),
  );
  $form['mass']['activate_all'] = array(
    '#type' => 'submit',
    '#value' => t('Activate all the PAD File'),
  );
  $form['mass']['delete_all'] = array(
    '#type' => 'submit',
    '#value' => t('Delete all the PAD File'),
  );
  $form['mass']['reset_counters'] = array(
    '#type' => 'submit',
    '#value' => t('Reset all the PAD File counters'),
  );

  return $form;
}

function _padfile_clear_cache() {
}

function _padfile_activate_all($status = 1) {
  db_query("UPDATE {padfile} SET active = %d", $status);
}

function _padfile_delete_all() {
  db_query("DELETE FROM {padfile}");
  db_query("DELETE FROM {padfile_descriptions}");
}

function _padfile_reset_counters() {
  db_query("UPDATE {padfile} SET download_counter = 0");
}

/**
 * Submit one of the Tools button.
 */
function padfile_form_tools_submit($form, &$form_state) {
  if (!$form_state['submitted']) {
    return;
  }

  if ($form_state['values']['op'] == $form_state['values']['clear_cache']) {
    _padfile_clear_cache();
    drupal_set_message('The PAD File was cleared.<br/>IMPORTANT NOTE: If you use Boost and cache XML files, then you also need to click that cache.');
  }
  elseif ($form_state['values']['op'] == $form_state['values']['desactivate_all']) {
    _padfile_activate_all(0);
    drupal_set_message('All the PAD Files were desactivated.');
  }
  elseif ($form_state['values']['op'] == $form_state['values']['activate_all']) {
    _padfile_activate_all(1);
    drupal_set_message('All the PAD Files are now active.');
  }
  elseif ($form_state['values']['op'] == $form_state['values']['delete_all']) {
    _padfile_delete_all();
    drupal_set_message('All the PAD Files were deleted.');
  }
  elseif ($form_state['values']['op'] == $form_state['values']['reset_counters']) {
    _padfile_reset_counters();
    drupal_set_message('All the PAD Files download counters were reset.');
  }
  else {
    // this should never happen
    drupal_set_message('Unknown action.', 'error');
  }
}

/**
 * Theme the PAD File overview as a sortable list of programs with version.
 *
 * @ingroup themeable
 * @see padfile_settings_list()
 */
function theme_padfile_content_list($form) {
  $has_page = module_exists('padfile_page');
  $rows = array();
  foreach (element_children($form) as $key) {
    if (isset($form[$key]['program_name'])) {
      $pad = &$form[$key]; // using & to avoid a copy

      $row = array();
      $row[] = drupal_render($pad['padid']);
      $row[] = drupal_render($pad['active']);
      $row[] = drupal_render($pad['program_name']);
      $row[] = drupal_render($pad['padfile_name']);
      $row[] = drupal_render($pad['version']);
      $row[] = drupal_render($pad['download_counter']);
      $row[] = drupal_render($pad['view']);
      $row[] = drupal_render($pad['export']);
      if ($has_page) {
        $row[] = drupal_render($pad['page']);
      }
      $row[] = drupal_render($pad['edit']);
      $row[] = drupal_render($pad['delete']);
      $rows[] = array(
        'data' => $row,
      );
    }
  }
  if (empty($rows)) {
    $rows[] = array(
      array(
        'data' => t('No PAD files available.'),
        'colspan' => 10 + ($has_page ? 1 : 0),
      ),
    );
  }
  else {
    if (variable_get('padfile_include_padring', 0)) {
      $padring_submit = '<br/>' . t('Go to !submit to submit your PADRING to the world.',
        array('!submit' => l('Submit to Pad Ring', 'http://www.padring.com/submit.html', array('attributes' => array('target' => '_blank')))));
    }
    else {
      $padring_submit = '';
    }
    $rows[] = array(
      array(
        'data' => t('You have !count PAD Files defined. View the !list of PAD Files.<br/>'
                  . 'Go to !validate to verify the validity of your PADFile.<br/>'
                  . 'Go to !submit to submit your PADFile to the world.',
          array(
            '!count' => count($rows),
            '!list' => l('list', 'padfile/list.xml', array('attributes' => array('target' => '_blank'))),
            '!validate' => l('PAD Online Validator', 'http://www.asp-shareware.org/pad/spec/validate.php', array('attributes' => array('target' => '_blank'))),
            '!submit' => l('Submit to PAD Repository', 'http://www.asp-shareware.org/pad/repository/user/submit.php?Home=user', array('attributes' => array('target' => '_blank'))),
          )
        ) . $padring_submit,
        'colspan' => 10 + ($has_page ? 1 : 0),
      ),
    );
  }

  $header = array(
    t('PADID'),
    t('Status'),
    t('Program'),
    t('Filename'),
    t('Version'),
    t('Downloads'),
    array(
      'data' => t('Operations'),
      'colspan' => 4 + ($has_page ? 1 : 0),
    ),
  );

  return theme('table', $header, $rows, array('id' => 'padfile')) . drupal_render($form);
}

/**
 * Theme the PAD File description overview as a sortable list of descriptions
 * with language and 45 chars description.
 *
 * @ingroup themeable
 * @see padfile_settings_list()
 */
function theme_padfile_content_description_list($form) {
  $has_page = module_exists('padfile_page');
  $rows = array();
  foreach (element_children($form) as $key) {
    if (isset($form[$key]['language'])) {
      $d = &$form[$key]; // using & to avoid a copy

      $row = array();
      $row[] = drupal_render($d['language']);
      $row[] = drupal_render($d['desc45']);
      $row[] = drupal_render($d['view']);
      $row[] = drupal_render($d['export']);
      if ($has_page) {
        $row[] = drupal_render($d['page']);
      }
      $row[] = drupal_render($d['edit']);
      $row[] = drupal_render($d['delete']);
      $rows[] = array(
        'data' => $row,
      );
    }
  }
  if (empty($rows)) {
    $rows[] = array(
      array(
        'data' => t('No description files available for the PAD file.'),
        'colspan' => 6 + ($has_page ? 1 : 0),
      ),
    );
  }
  else {
    $rows[] = array(
      array(
        'data' => t('You have !count descriptions defined.', array('!count' => count($rows))),
        'colspan' => 6 + ($has_page ? 1 : 0),
      ),
    );
  }

  $header = array(
    t('Language'),
    t('Description'),
    array(
      'data' => t('Operations'),
      'colspan' => 4 + ($has_page ? 1 : 0),
    ),
  );

  return theme('table', $header, $rows, array('id' => 'padfile_descriptions')) . drupal_render($form);
}

// vim: ts=2 sw=2 et syntax=php
